/*
 * Copyright 2005, Ross Werner.
 * Distributed under the terms of the GPL version 2.
 * See LICENSE.TXT for details.
 */

package org.padict.pdb;

/**
 * @author ross
 */
public class PdbRecord {
	private byte attributes;
	private int uniqueID;
	private byte[] data;
	
	public PdbRecord(byte[] data, byte attributes, int uniqueID) {
		this.data = data;
		this.attributes = attributes;
		this.uniqueID = uniqueID;
	}
	
	public int getSize() {
		return data.length;
	}
	
	public int getId() {
		return uniqueID;
	}
	
	public void setId(int newID) {
		uniqueID = newID;
	}
	
	public String getDataString() {
		if (data.length == 0) return "";
		if (data[data.length - 1] != 0x00) return null;
		
		for (int i = 0; i < (data.length - 1); i++) {
			if (data[i] < ' ' || data[i] > '~') {
				if (data[i] != '\n') {
System.out.println("Unprintable character discovered: " + (char)data[i] + " (0x" + Integer.toHexString(data[i]) + ")");				
					return null;
				}
			}
		}
		
		return new String(data, 0, data.length - 1);
	}
	
	public void setData(String newData) {
		if (newData.equals("")) {
			data = new byte[0];
		} else {
			byte[] buffer = newData.getBytes();
			data = new byte[buffer.length + 1];
			for (int i = 0; i < buffer.length; i++)
				data[i] = buffer[i];

			data[buffer.length] = 0x00; // Null terminate String
		}
	}
	
	public byte[] getData() {
		return data;
	}
	
	public byte getAttributes() {
		return attributes;
	}
	
	public void setAttributes(byte newAttributes) {
		attributes = newAttributes;
	}
	
	public boolean isSecret() {
		return (attributes & 0x10) == 0x10;
	}
	
	public boolean isBusy() {
		return (attributes & 0x20) == 0x20;
	}
	
	public boolean isDirty() {
		return (attributes & 0x40) == 0x40;
	}
	
	public boolean isDeleted() {
		return (attributes & 0x80) == 0x80;
	}
	
	public int getCategory() {
		return (int)(attributes & 0x0f);
	}
	
	
	/* Debug Methods */
	
	public String toString() {
		String toReturn = "";
		
		toReturn += "uniqueID:             " + Integer.toHexString(uniqueID) + "\n";
		toReturn += "attributes:           ";
		
		for (int i = 0; i < 8; i++) {
			toReturn += (attributes & (0x1 << i)) == (0x1 << i) ? "x" : ".";
		}
		toReturn += "\n";
		
		toReturn += getHexDump();
		
		return toReturn;
	}
	
	private String getHexDump() {
		String toReturn = "";
		
		for (int i = 0; i < data.length; i += 16) {
			for (int j = 0; j < 16; j++) {
				if (j == 8) toReturn += " ";
				if (i + j < data.length) {
					int b = data[i + j] & 0xff;
					toReturn += ((b < 0x10) ? "0" : "");
					toReturn += Integer.toHexString(b);
				} else {
					toReturn += "  ";
				}
				if ((i + j) % 2 == 1) toReturn += " ";
			}
			
			for (int j = 0; j < 16; j++) {
				if (j == 8) toReturn += " ";
				if (i + j < data.length) {
					if (data[i + j] >= ' ' && data[i + j] <= '~') {
						toReturn += (char)(data[i + j]);
					} else {
						toReturn += ".";
					}
				}
			}
			
			toReturn += "\n";
		}
		
		return toReturn;
	}
}